########################
### Dimension générique de l'espace des modules 
########################

load("Saito.sage")

def ModHom(n):
	if mod(n,2)==0:
		return((n-2)^2/4)
	else:
		return((n-1)*(n-3)/4)


def sigma(x):
	return((x-1)*(x-2)/2)

def DimensionCoho(m,dic,delta):
	if dic==0:
		return(sigma(ceil(m/2)-1)+sigma(m-ceil(m/2))+m-2-delta)
	else:
		return(sigma(floor(m/2))+sigma(m-floor(m/2)))

############# Generic Dimension of the moduli space
def DimensionModGen(M,nu):
	ValCourbe=Valuation(M,nu)
	if M==[[1]]:
		return(ModHom(nu[0][1]))
	else:
		Resultat=Saito(M,nu)
		ValSaito=ValuationFeuilletage(M,[l[1:3] for l in Resultat])
		Res=DimensionCoho(ValCourbe,Resultat[0][1],ValSaito-Resultat[0][2])
		DecompEcla=EclatementDecomposition(M,[l[0] for l in nu])	
		nuu=deepcopy(nu)
		for i in range(0,len(DecompEcla[2])):					
			Newnu=ExtraireValuation(DecompEcla[1][i],nuu)
			for j in range(0,len(Newnu)):
				if Newnu[j][0]==DecompEcla[2][i]:
						Newnu[j][1]=Newnu[j][1]+1		
			Res=Res+DimensionModGen(DecompEcla[0][i],Newnu)
		return(Res)
			


########## Compute the Milnor number
def Milnor(M,nu):
	ValCourbe=Valuation(M,nu)
	if M==[[1]]:
		return((ValCourbe-1)^2)
	else:
		Res=(ValCourbe-1)^2-(ValCourbe-1)-1+nu[0][1]		
		DecompEcla=EclatementDecomposition(M,[l[0] for l in nu])	
		nuuu=deepcopy(nu)
		for i in range(0,len(DecompEcla[2])):					
			Newnu=ExtraireValuation(DecompEcla[1][i],nuuu)
			for j in range(0,len(Newnu)):
				if Newnu[j][0]==DecompEcla[2][i]:
						Newnu[j][1]=Newnu[j][1]+1		
			Res=Res+Milnor(DecompEcla[0][i],Newnu)
		return(Res)
		
############# Compute the Modularity number
def Modularity(M,nu):
	ValCourbe=Valuation(M,nu)
	if M==[[1]]:
		return((ValCourbe-2)*(ValCourbe-3)/2)
	else:
		Res=(ValCourbe-2)*(ValCourbe-3)/2		
		DecompEcla=EclatementDecomposition(M,[l[0] for l in nu])	
		nuuuu=deepcopy(nu)
		for i in range(0,len(DecompEcla[2])):					
			Newnu=ExtraireValuation(DecompEcla[1][i],nuuuu)
			for j in range(0,len(Newnu)):
				if Newnu[j][0]==DecompEcla[2][i]:
						Newnu[j][1]=Newnu[j][1]+1	
			Res=Res+Modularity(DecompEcla[0][i],Newnu)
		return(Res)







############# Summarize the various invariants
def Invariants(M,nu):
	ValCourbe=Valuation(M,nu)
	MilnorNumber=Milnor(M,nu)
	ModularityNumber=Modularity(M,nu)
	DimensionGen=DimensionModGen(M,nu)
	TopologyX1=Saito(M,nu)
	ValuationSaito=ValuationFeuilletage(M,[l[1:3] for l in TopologyX1])
	print('Results on the Saito Modules')
	print('----------------------------')
	print('Valuation of the curve = ', ValCourbe)
	print('Topology of the Saito foliation = ', TopologyX1)
	if TopologyX1[0][1]==0: 
		print('The Saito foliation is dicritical and there are ',TopologyX1[0][2],' free points.')
	else:
		print('The Saito foliation is not dicritical.')
	print('Valuation of the Saito foliation = ',ValuationSaito)	 
	print('----------------------------')
	print('Basic invariants')
	print('---------------')
	print('Milnor number = ', MilnorNumber)
	print('Modularity number = ', ModularityNumber)
	print('Generic dimension of the moduli space = ', DimensionGen)	
	print('Minimum Tjurina number = ', MilnorNumber-ModularityNumber+DimensionGen)



####### Exemples : x=t^8, y=t^20+t^30+t^35

M20=[
[1, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
[0, 1, -1, -1, 0, 0, 0, 0, 0, 0, 0, 0],
[0, 0, 1, -1, 0, 0, 0, 0, 0, 0, 0, 0],
[0, 0, 0, 1, -1, 0, 0, 0, 0, 0, 0, 0],
[0, 0, 0, 0, 1, -1, 0, 0, 0, 0, 0, 0],
[0, 0, 0, 0, 0, 1, -1, -1, 0, 0, 0, 0],
[0, 0, 0, 0, 0, 0, 1, -1, 0, 0, 0, 0],
[0, 0, 0, 0, 0, 0, 0, 1, -1, 0, 0, 0],
[0, 0, 0, 0, 0, 0, 0, 0, 1, -1, 0, 0],
[0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -1, -1],
[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -1],
[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]]

Nu20=[[0, 0],
 [1, 0],
 [2, 0],
 [3, 0],
 [4, 0],
 [5, 0],
 [6, 0],
 [7, 0],
 [8, 0],
 [9, 0],
 [10, 0],
 [11, 1]]


####### Exemples deux composantes x^3-y^5 transverses avec I=9

M21=[
[1, -1, -1, 0, -1, -1, 0],
[0, 1, -1, -1, 0, 0, 0],
[0, 0, 1, -1, 0, 0, 0],
[0, 0, 0, 1, 0, 0, 0],
[0, 0, 0, 0, 1, -1, -1],
[0, 0, 0, 0, 0, 1, -1],
[0, 0, 0, 0, 0, 0, 1]]

Nu21=[[0, 0],
 [1, 0],
 [2, 0],
 [3, 1],
 [4, 0],
 [5, 0],
 [6, 1]]


####### Exemples deux composantes x^3-y^5 tangentes avec I=15

M22=[
[1, -1, -1, 0],
[0, 1, -1, -1],
[0, 0, 1, -1],
[0, 0, 0, 1]]

Nu22=[[0,0],[1,0],[2,0],[3,2]]



####### Exemples deux composantes x^3-y^5 tangentes avec I=16

M23=[
[1, -1, -1, 0,0],
[0, 1, -1, -1,0],
[0, 0, 1, -1,0],
[0, 0, 0, 1,-1],
[0, 0, 0, 0,1]]

Nu23=[[0,0],[1,0],[2,0],[3,0],[4,2]]

